C TEMP, IN CENTIGRADE, PARABOLIC RATE CONSTANT , UNITS BELOW
C ALLOT START T, ALLOT FINISH TEMP (CENTIGRADE) L (MICRONS) VMAX (MICRONS/S)
C X DATA MUST BE IN ASCENDING ORDER
C IF G=X/XMAX, Y=A0+A1*GA+A2*G*G....
C PARABOLIC RATE CONSTANT READ IN UNITS OF (CM /ROOT SEC)/D-03
C L READ IN MICRONS, EQUAL TO TWICE HEXAGON SIDE LENGTH
C I1 IS NUMBER OF ALPHA1 VALUES, I7 IS DEGREE OF POLYNOMIAL
C
C Reference: Bhadeshia et al, 1985, Acta metall., vol. 33, pp1271-1283.
C
      SUBROUTINE MAP_STEEL_AN2(X,Y,HIGHT,LOWT,L,VMAX,C1,C2,TINT,CURR,
     & VOLT,EFF,VELOC,JI3,VACIC,VOLW,VOLFRA,Q,TIMC,PPMBSL,JBOR,FBORW)
C
      IMPLICIT NONE
      DOUBLE PRECISION ANS,BETA,C1,C2,C4,C7,C8,CC,CURR,DELTA,EFF,FBORW
      DOUBLE PRECISION FTIM,HIGHT,L,LOWT,PPMBSL,Q,THETA,TIMC,TIME,TINT
      DOUBLE PRECISION V1,V3,V8,VACIC,VELOC,VMAX,VOLFRA,VOLT,VOLW,VSUM
      DOUBLE PRECISION A(10),SEC(50),X(6),Y(6),YY(50)
      INTEGER I1,I2,I6,I10,I7,I11,JI3,JBOR
C
      I1 = 6
      I7 = 3
C
C Experimental volume fraction of allotriomorphic ferrite = 
C              C7 * (calculated value) + C8      (equ. 6(b) of reference)
C
      IF (JI3 .EQ. 5) GOTO 100 
      C7 = 2.041511D0
      C8 = 0.034729D0
      GOTO 110
C
C If JI3 = 5 then do not make any correction:
C
  100 C7   = 1.0D0
      C8   = 0.0D0
C
  110 VMAX = VMAX*1.0D6
      V1   = X(I1)
      DO 120 I2=1,I1
         Y(I2) = Y(I2)/1.0D-03
         X(I2) = X(I2)/V1
  120 CONTINUE
C
C Express the parabolic rate constant data (Y) as a polynomial of order I7 
C The polynomial is a function of temperature.
C
      CALL MAP_UTIL_POLY(X,Y,I1,A)
      DO 140 I6=1,I1
         V3 = 0.0D0
         DO 130 I10=2,I7
            V3 = V3+A(I10)*(X(I6)**(I10-1))
  130    CONTINUE
         Y(I6) = V3+A(1)
         X(I6) = X(I6)*V1
  140 CONTINUE
C
      THETA = C1*(C2-1.0D0)/(CURR*VOLT*EFF/VELOC)
      BETA  = (HIGHT-TINT)**(1.0D0-C2)
      DELTA = 1.0D0/(1.0D0-C2)
C
C Find time taken (TIME) for alloy to cool from HIGHT to LOWT
C
      CALL MAP_STEEL_TIM(C1,C2,TIME,HIGHT,LOWT,CURR,VOLT,EFF,VELOC,TINT)
C
C Calculate the half thickness q (Q) of allotriomorphic ferrite obtained by
C continuous cooling from HIGHT to LOWT over time TIME using equation 5. The 
C temperature in the polynomial for the parabolic rate constant is reexpressed
C in terms of time using equation 2(b).
C
      FTIM = 0.0D0
      DO 150 I11=1,100
         FTIM = FTIM + 0.05*TIME
         SEC(I11) = FTIM
         IF (FTIM .GT. 1.0001*TIME) GOTO 160
         V8 = 1.0D0/(DSQRT(FTIM))
         YY(I11)=A(1)*V8+(A(2)/V1)*V8*(((THETA*FTIM+BETA)**DELTA)+TINT)
     &           +((A(3)*V8)/(V1*V1))*
     &           ((((THETA*FTIM+BETA)**DELTA)+TINT)**2.0D0)
         YY(I11)=YY(I11)*0.5D0
  150 CONTINUE
  160 I11 = I11-1
      CALL MAP_UTIL_TRAPE(SEC,YY,ANS,I11)
      Q = 10.D0*ANS
C
C For details on C4 and TIMC see p32 of ESAB project book.
C C4 is a constant. Found to be 6.67 /s. 
C TIMC is maximum time available for unrestricted Widmanstatten growth. It 
C was found to be 0.21 s.
C
      TIMC = 0.211197871D0
      C4   = 6.871754682D0
C
C Adjust C4 to allow for the effect of boron on Widmanstatten ferrite.
C
      IF (JBOR .EQ. 1) C4 = 6.871754682D0-PPMBSL*FBORW
      IF (C4 .LT. 0.0D0) C4 = 0.0D0
      IF (JI3 .EQ. 5) GOTO 170
      VOLFRA = 4.61880216D0*Q*(L-1.15470054D0*Q)/(L*L)
C
C Apply fudge factor of 2.041511 to correct VOLFRA to experimental value.
C q' (Q) value therefore modified from calculated value.
C
      VOLFRA = C7*VOLFRA + C8
C
C 17 March 1990
C
      IF (VOLFRA .LT. 0.0D0 .OR. VOLFRA .GT. 1.0D0)
     &    VOLFRA = 0.999D0
C
C Calculate a corrected value q' for the allotriomorphic ferrite half-thickness 
C using equation 6(c). 0.577350=TAN(30 deg.)
C
  170 Q = (L-DSQRT(L*L-VOLFRA*L*L))/(4.0D+00*0.577350)
C
C  0.034729=fudge factor (p47 of ESAB project book)
C
C Calculate the time required for for a Widmanstatten ferrite plate to grow
C unhindered across an austenite grain using equation 11.
C
      CC = (0.8660254D0*L-2.0D0*Q)/VMAX
      IF (TIMC .LT. CC) GOTO 180
      GOTO 190
C
C Acicular ferrite formation interferes with Widmanstatten growth. 
C Widmanstatten plates can only grow to length VMAX*TIMC.
C Calculate volume fraction of Widmanstatten using equation 8.
C
  180 VOLW = C4*VMAX*TIMC*TIMC*(L-2.3094D0*Q)/(L*L)
      GOTO 200
C
C Acicular ferrite formation does not interfere with Widmanstatten growth.
C Time available for growth = CC. 
C Calculate volume fraction of Widmanstatten using equation 8.
C
  190 TIMC = (0.8660254D0*L-2.0D0*Q)/VMAX
      VOLW = C4*VMAX*TIMC*TIMC*(L-2.3094D0*Q)/(L*L)
C
C When VOLFRA is large, VOLW calculation can be wrong with the sum of
C VOLFRA and VOLW exceeding unity. 17 March 1990. Therefore:
C
  200 IF (VOLFRA .LE. 1.0D0 ) GOTO 210
      VOLFRA = 1.0D0
      GOTO 220
  210 VSUM = VOLFRA+VOLW
      IF (VSUM .LT. 1.0D0) GOTO 230
  220 VOLW  = 1.0D+00-VOLFRA
  230 VACIC = 1.0D+00-VOLFRA-VOLW
      RETURN
      END
